@@ -2,8 +2,11 @@ package SVK::Command;
 use strict;
 our $VERSION = '0.09';
 use Getopt::Long qw(:config no_ignore_case);
-use Pod::Text;
-use File::Find;
+# XXX: Pod::Simple isn't happy with SVN::Simple::Edit, so load it first
+use SVN::Simple::Edit;
+use Pod::Simple::SimpleTree ();
+use Pod::Text ();
+use File::Find ();
 use Cwd;
 
 my %alias = qw( co checkout
@@ -29,7 +32,7 @@ sub new {
     bless {}, $class;
 }
 
-sub options {}
+sub options { () }
 
 sub parse_arg {}
 
@@ -62,26 +65,43 @@ sub help {
 	my $dir = $INC{'SVK/Command.pm'};
 	$dir =~ s/\.pm$//;
 	print "Available commands:\n";
-	find (sub {
-		  push @cmd, lc($_) if s/\.pm$//;
-	      }, $dir);
-	print "  $_\n" for sort @cmd;
+	File::Find::find (sub {
+			      push @cmd, $File::Find::name if m/\.pm$/;
+			  }, $dir);
+	$pkg->brief_usage ($_) for sort @cmd;
 	return;
     }
-    $cmd = get_cmd ($pkg, $cmd);
-    $cmd->usage (1);
+    get_cmd ($pkg, $cmd)->usage(1);
 }
 
 sub invoke {
     my $pkg = shift;
-    my $info = shift;
+    my $xd = shift;
     my $cmd = shift;
     local @ARGV = @_;
 
     $cmd = get_cmd ($pkg, $cmd);
-    $cmd->{info} = $info;
+    $cmd->{xd} = $xd;
     die unless GetOptions ($cmd, _opt_map($cmd, $cmd->options));
-    $cmd->run ($cmd->parse_arg(@ARGV));
+    my @args = $cmd->parse_arg(@ARGV);
+    $cmd->lock (@args);
+    my $ret = $cmd->run (@args);
+    $xd->unlock ();
+    return $ret;
+}
+
+sub brief_usage {
+    my ($self, $file) = @_;
+    my $fname = ref($self);
+    $fname =~ s|::|/|g;
+    my $parser = Pod::Simple::SimpleTree->new;
+    my @rows = @{$parser->parse_file($file || $INC{"$fname.pm"})->root};
+    while (my $row = shift @rows) {
+        if ( ref($row) eq 'ARRAY' && $row->[0] eq 'head1' && $row->[2] eq 'NAME')  {
+            print "\t". $rows[0]->[2]."\n";
+            last;
+        }
+    }
 }
 
 sub usage {
@@ -95,6 +115,19 @@ sub usage {
     exit 0;
 }
 
+sub lock_target {
+    my ($self, $target) = @_;
+    $self->{xd}->lock ($target->{copath});
+}
+
+sub lock_none {
+    my ($self) = @_;
+    $self->{xd}->giant_unlock ();
+}
+
+sub lock {
+}
+
 sub arg_condensed {
     my ($self, @arg) = @_;
     $self->usage if $#arg < 0;
@@ -113,10 +146,11 @@ sub arg_condensed {
 sub arg_co_maybe {
     my ($self, $arg) = @_;
 
-    my ($repospath, $path, $copath, undef, $repos) =
+    my ($repospath, $path, $copath, $cinfo, $repos) =
 	main::find_repos_from_co_maybe ($arg, 1);
     return { repos => $repos,
 	     repospath => $repospath,
+	     depotpath => $cinfo->{depotpath} || $arg,
 	     copath => $copath,
 	     path => $path,
 	   };
@@ -147,6 +181,12 @@ sub arg_depotpath {
 	   };
 }
 
+sub arg_depotname {
+    my ($self, $arg) = @_;
+
+    return main::find_depotname ($arg, 1);
+}
+
 sub arg_path {
     my ($self, $arg) = @_;
 
