@@ -1,9 +1,8 @@
 package SVK::Command::Cat;
 use strict;
-our $VERSION = '0.09';
+our $VERSION = '0.11';
 
 use base qw( SVK::Command );
-use SVK::XD;
 use SVK::Util qw(slurp_fh);
 
 sub options {
@@ -11,11 +10,12 @@ sub options {
 }
 
 sub parse_arg {
-    my $self = shift;
-    my @arg = @_;
+    my ($self, @arg) = @_;
     return @arg;
 }
 
+sub lock { $_[0]->lock_none }
+
 sub run {
     my ($self, @arg) = @_;
     for (@arg) {
@@ -36,3 +36,30 @@ sub run {
 
 1;
 
+=head1 NAME
+
+cat - Output the file from depot.
+
+=head1 SYNOPSIS
+
+    cat [DEPOTPATH|PATH...]
+
+=head1 OPTIONS
+
+    options:
+    -r [--revision] REV:    revision
+
+=head1 AUTHORS
+
+Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
+
+=head1 COPYRIGHT
+
+Copyright 2003-2004 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
+
+This program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
+
+See L<http://www.perl.com/perl/misc/Artistic.html>
+
+=cut
